#!/usr/bin/python

#
# File:	jbm_detect.py
#
# Description: Provides device auto detection for Jon's Broadband
#				Manager
#
# License:
#		jbm_detect.py
#		Copyright 2009 Jon Estey
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License version 2 as
#		published by the Free Software Foundation
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#

# **************** Import ****************
from subprocess import *
import serial
import time

# **************** Definitions *****************

# USB Detect Return Codes
UNKNOWN		= 0x0000

CD_MODE		= 0x0001 # Device is in cd mode
MODEM_MODE	= 0x0002 # Device is in modem mode

MODEM_MASK	= 0xFF00 # Mask for modem values
MF626		= 0x0100 # Device is an MF626

# **************** Functions ****************
def usb_detect():
	"""Detects if the modem is connected to the USB port and if it
	is in modem mode or automount cd mode
	
	Returns:
		See USBDetect Return Flags
	"""
	
	# Run lsusb
	lsusb = Popen("lsusb", shell=True, bufsize=0, stdout=PIPE, stderr=STDOUT)
	(stdout, stderr) = lsusb.communicate()
	
	# Split output into lines
	lines = stdout.split("\n")
	
	# Loop through lines
	for line in lines:
		# Clean up the line
		line = line.strip()
		
		# Skip blank lines
		if line == "":
			continue
			
		# Try to split off the vendor and device
		try:
			(vendor, device) = line[-9:].split(":")
		except ValueError:
			# Bad line, skip
			continue
			
		# Process the vendor and device to see if it is a known product
		retval = process_device(vendor, device)
		
		# Check to see if a device was found
		if not retval == UNKNOWN:
			# Stop checking
			return retval
	
	# No valid device found return unknown..
	return UNKNOWN


def process_device(vendor, device):
	"""Checks the provided device and vendor against known values"""
	
	if vendor == "19d2":
		# ZTE vendor ID
		if device == "0031":
			# MF626/628/636 USB Modem
			return MF626|MODEM_MODE
		elif device == "2000":
			# MF626/628/626 USB Modem, CD Mode
			return MF626|CD_MODE
	
	return UNKNOWN
	

def scan_tty():
	"""Scans serial ports trying to determine proper port for dialing
	and commands
	
	Returns a tuple (command_tty, dial_tty)
	"""
	
	# Get a list of valid serial ports
	lsdev = Popen("ls /dev | grep ttyUSB", shell=True, bufsize=0, stdout=PIPE, stderr=STDOUT)
	(stdout, stderr) = lsdev.communicate()
	del lsdev
	
	# Split up list
	ports = stdout.split()
	
	# Create a list to hold valid ports
	valid_ports = []
	
	# Loop through valid ports
	for port in ports:
		# Set the port
		port = "/dev/" + port
		
		# Create a tty object
		tty = serial.Serial()
			
		# Setup the serial port communications
		tty.baudrate = 9600
		tty.bytesize = serial.EIGHTBITS
		tty.parity = serial.PARITY_NONE
		tty.stopbits = serial.STOPBITS_ONE
		tty.timeout = 0.1
		tty.xonxoff = 0
		tty.rtscts = 0
		tty.interCharTimeout = None
		tty.port = port
		
		print "Checking port {0}".format(tty.port)
		
		# Check the port
		retval = check_port(tty)
		
		# Delete the tty
		del tty
		
		if retval == 1:
			# Valid port found, remember it
			valid_ports.append(port)

			# Stop checking after second port is found
			if len(valid_ports) > 1:
				break
		
	# Return ports
	if len(valid_ports) > 1:
		# Default first port to being commands and second to being dialer
		return (valid_ports[0], valid_ports[1])
	else:
		# Only return valid ports if two are found
		return (None, None)

def check_port(tty):
	"""Opens serial object and trys to send a couple of commands
	
	Returns, -1 if an error occurs, 0 for an invalid port, or 1 for a
	valid port
	"""
	
	# Open the port
	try:
		tty.open()
	except serial.SerialException:
		# Error Opening port...
		print "Could not open!"
		return -1
		
	# Try to send a command
	try:
		tty.write("AT\r\n")
	except:
		# Error...
		print "Could not write!"
		tty.close()
		return -1
	
	# Allow 1/4 a second for the proper responce
	timeout = time.time() + 0.25
	
	# Keep going until either ok is found or a timeout
	while time.time() < timeout:
		# Read a line
		line = tty.readline()
		
		# Clean up the line
		line = line.strip()

		# Check for OK
		if line == "OK":
			# Close the port
			tty.close()
			
			# Return 1, the port responded to the command
			return 1
	
	# Close the port
	tty.close()
	
	# Return false, no valid responce was found
	return 0
	
	
def modeswitch(modem):
	"""Runs usb_modeswitch for the specified modem
	
	Parameters:
		modem, Value returned by process device
	"""
	
	# Initialize variables
	ddev = ""
	dven = ""
	tdev = ""
	tven = ""
	msg = ""
	

	# Check to see if the modem is already in modem mode
	if ((modem & MODEM_MODE) > 0):
		return True
	
	# Get only modem values
	tmpmodem = (modem & MODEM_MASK)
		
	# Check for modem model
	if tmpmodem == MF626:
		ddev = "0x2000"
		dven = "0x19d2"
		tdev = "0x0031"
		tven = dven
		msg = "55534243123456782000000080000c85010101180101010101000000000000"
		
	
	# Check to see if info was found for the modem
	if msg == "":
		# Run usb_modeswitch with no options
		command = "usb_modeswitch"
	else:
		# Run usb_modeswitch with found options
		command = "usb_modeswitch -v {0} -p {1} -V {2} -P {3} -M {4}".format(
				dven, ddev, tven, tdev, msg)
	

	# Run mode switch
	modeswitch = Popen(command, shell=True, bufsize=0, stdout=PIPE, stderr=STDOUT)
	(stdout, stderr) = modeswitch.communicate()
	del modeswitch
	
	# Process the output
	lines = stdout.split("\n")
	for line in lines:
		if line == " OK, message successfully sent":
			return True
		
	return False
		
# **************** Main ****************
if __name__ == "__main__":
	# Test code
	
	# Scan for devices
	retval = usb_detect()
	
	if retval == UNKNOWN:
		# No valid device
		print "Nothing valid..."
		exit()
		
	if (retval & MODEM_MODE) > 0:
		# Device found and its ready to go, scan serial ports
		print "Scanning...."
		command, dial = scan_tty()
		print "Found: {0}, {1}".format(command, dial)
		
	if (retval & CD_MODE) > 0:
		# Device is in CD mode, run usb_modeswitch
		print "Run USB Mode switch!!"
		exit()
	
	
