#!/usr/bin/python

#
# File:	jbm_usage.py
#
# Description: Module used for recording data usage for the Jon's
#				Broadband Manager
#
# License:
#		jbm_usage.py
#		Copyright 2009 Jon Estey
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License version 2 as
#		published by the Free Software Foundation
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

# **************** Imports **************
import os
from datetime import datetime
import time
from threading import *

# **************** Classes **************
class JBMUsage():
	"""Main class for accessing wireless broadband usage data"""
	
	class Record():
		"""Class that processes and holds usage records"""
		__slots__ = ("index","date","duration","up","down","total","from_file")
		
		def __init__(self, data_line=""):
			"""Initiates the class"""
			
			# Process the data line if there is one
			if not data_line == "":
				data_line = data_line.strip()
				
				data = data_line.split()
				
				self.index = long(data[0])
				self.date = datetime.fromtimestamp(float(data[1]))
				self.duration = long(data[2])
				self.up = long(data[3])
				self.down = long(data[4])
				self.from_file = True
			else:
				# Set defaults
				self.index = 0
				self.date = datetime.now()
				self.duration = 0
				self.up = 0
				self.down = 0				
				self.from_file = False
		
			self.total = self.up + self.down
			
		def __str__(self):	
			"""Returns the string representation of the object"""
			
			return "{0} {1} {2} {3} {4}".format(self.index, time.mktime(self.date.timetuple()), self.duration, self.up, self.down)
			
		def add(self, record):
			"""Adds the supplied records information to the current record"""
			
			self.duration += record.duration
			self.up += record.up
			self.down += record.down
			self.total += record.total
			
		def same_year(self, record):
			"""Returns true if the supplied record and current record are
			from the same year"""
			
			return (self.date.year == record.date.year)
			
		def same_month(self, record):
			"""Returns true if the supplied record and current record are
			from the same month"""
			
			if not self.same_year(record):
				return False
				
			return (self.date.month == record.date.month)
			
		def same_day(self, record):
			"""Returns true if the supplied record and current record are
			from the same day"""

			if not self.same_month(record):
				return False
				
			return (self.date.day == record.date.day)
	
	
	def __init__(self, data_file, config, load=True):
		"""Initializes the class
		
		Parameters:
			data_file, the path and file name where usage data
				should be/is stored
				
			config, confiuration object where settings are stored
			
			load, if True the usage data is automatically loaded
		"""
		
		self.file = data_file

		# Load config settings
		self.rollup = config.get("rollup days","usage",True)
		self.short_format = config.get("short format","usage","%d-%b-%Y")
		self.long_format = config.get("long format","usage","%d-%b-%Y %H:%M")
		
		# Setup variables
		self.data = []
		self.current = None
		self.sub_total = None
		self.interface = ""
		
		# Get today's date if rolling should occur
		if self.rollup:
			self.today = datetime.today()
		
		# Load the data file if requested
		if load:
			self.load()
			
	def __iter__(self):	
		"""The usagelog itself is an iterator, returns self"""
		self.iter_pos = -1
		return self
	
	def __get_next(self):
		"""Used for iteration, returns next valid record or None if at
		at the end of the data list"""
		
		# Increment the data pointer
		self.iter_pos += 1

		# Check to make sure that the end of the data hasn't been found
		if self.iter_pos >= len(self.data):
			return None
		
		# Check make sure the record isn't the current data
		if self.current == self.data[self.iter_pos]:
			# Return the next valid record
			return self.__get_next()
			
		# Return the current record
		return self.data[self.iter_pos]
	
	def __roll_records(self, first, comp_func):
		"""Rolls up records using first as a base and comp_func to
		compare it with the next record"""
		
		# Copy the first record into a temporary record
		rolled_rec = JBMUsage.Record(str(first))
		
		# Continue getting the next record until either the comparison
		#	is false or the end of the data is found
		record = self.__get_next()
		while not record == None:
			# Check to see if the record should be rolled
			if not comp_func(record):
				break
			
			# Add the value to the temp record
			rolled_rec.add(record)
			
			# Get the next record
			record = self.__get_next()
		
		# Scale back the iteration position by one
		self.iter_pos -= 1
		
		# Return the temporary record
		return rolled_rec
		
	def next(self):
		"""Used for iteration, returns the next data section"""
	
		
		# Check to see if the usage should be rolled up
		if self.rollup:
			# Rollup old data
			
			# Get the next record
			tmp_rec = self.__get_next()
		
			# Check to see if iteration should stop
			if tmp_rec == None:
				raise StopIteration
			
			# Check the records date to see if rolling should occur
			if tmp_rec.date.year >= self.today.year:
				if tmp_rec.date.month >= self.today.month:
					if tmp_rec.date.day >= self.today.day:
						# No rolling should occur, return the record
						record = tmp_rec
						short = False
					else:
						# Day rolling should occur
						record = self.__roll_records(tmp_rec, tmp_rec.same_day)
						short = True
						
				else:
					# Month rolling should occur
					record = self.__roll_records(tmp_rec, tmp_rec.same_month)
					short = True
			else:
				# Year rolling should occur
				record = self.__roll_records(tmp_rec, tmp_rec.same_year)
				short = True
		else:
			# Return all data
			
			short = False
			
			# Get the next record
			record = self.__get_next()
			
			# Check to see if any data is left
			if record == None:
				raise StopIteration		
				
		# Format the record
		rec_form = self.format_record(record,short)

		# Return the formated record
		return rec_form
	
		
	def format_size(self, size):
		"""Processes the given raw size and returns the value as a
		a string. For example passing 2024 will return 1.97kB"""
	
		if size > 1073741824:
			# Value is between greater then 1GB, return value in GB
			return "{0:.4g}GB".format(float(size)/1073741824)
		elif size > 1048576:
			# Value is between 1GB and 1kB return value in MB
			return "{0:.4g}MB".format(float(size)/1048576)
		elif size > 1024:
			# Value is between 1MB and 1kB return value in kB
			return "{0:.4g}kB".format(float(size)/1024)
			
		# Value is less then 1kB, return value as Bytes
		return "{0:.4g}B".format(size)
	
	def format_duration(self, usage):
		"""Formats the usage into H:MM:SS format"""
		
		# Calculate the hours miniutes and seconds
		hours = int(usage/3600)
		usage = usage - (hours*3600)
		mins = int(usage/60)
		secs = usage - (mins*60)
		
		# Return the string representation
		return "{0}:{1:02}:{2:02}".format(hours, mins, secs)	
		 
	def format_date(self, date, short=False):
		"""Returns the specifed date formated accordingly, either in 
		short form if short is True or long if short is False"""
		
		if short:
			return date.strftime(self.short_format)
		else:
			return date.strftime(self.long_format)
	
	def format_record(self, record, short=False):
		"""Returns a list representing the formated record, if short is
		True record is formated as a rolled up record"""
		
		# Create a temporary list
		rec = [0]*5
		
		# Format the record sections
		rec[0] = self.format_date(record.date, short)
		rec[1] = self.format_duration(record.duration)
		rec[2] = self.format_size(record.up)
		rec[3] = self.format_size(record.down)
		rec[4] = self.format_size(record.total)
		
		# Return the list
		return rec
		
	def load(self):
		"""Loads the usage data from the specified data file"""
		
		# Open the usage log if it exists
		if not os.path.exists(self.file):
			return
			
		idlog = open(self.file, "r")
		
		# Read the log
		for line in idlog:
			# Clean up the line
			line = line.strip()
			
			# Check for comments
			if line[0] == "#":
				continue
			
			# Process the line
			record = JBMUsage.Record(line)
			
			# Add the record to the data
			self.data.append(record)

		# Close the usage log
		idlog.close()
	
		# Clear the sub total
		self.sub_total = None
		
	def save(self):
		"""Save the usage log"""
		
		try:
			# Check to see if the log exists
			if not os.path.exists(self.file):
				# Log doesnt exists try to create it
				idlog = open(self.file, "w")
				
				# Write initial comments
				idlog.write("# Generated by Jon's Broadband Manager\n")
			else:
				# Open the log in append mode to add new records
				idlog = open(self.file, "a")
		except IOError:
			# Report the error
			print "Error opening usage log, usage data can't be saved"
			return
			
		# Loop through the items in the record data
		for record in self.data:
			# Only append records that arn't already in the usage log
			if not record.from_file:
				# Write the record
				idlog.write(str(record) + "\n")
				
				# Change the from_file flag to indicate the record has
				#	been saved
				record.from_file = True
		
		# Close the usage log
		idlog.close()
	
	def __update_current(self):
		"""Updates the current usage from the system"""
		
		# Check to see if there is a current connection
		if self.current == None:
			return
		
		# Set the stats path
		stats_path = "/sys/class/net/{0}/statistics/".format(self.interface)
		
		# This is done this way so that if the interface is
		#	closed before the update is processed the current data isn't
		#	completely lost by reseting the current values to zero
		
		# Update the duration
		cur_time = datetime.now()
		dur = cur_time - self.current.date
		self.current.duration = dur.seconds

		# Try to open the recieved data
		try:
			with open(stats_path + "rx_bytes") as file:
				# Try to read some data
				data = file.readline()
				
				# Check to see if data was read
				if not data == "":
					# Update the recieved bytes
					self.current.down = int(data.strip())
		except IOError:
			pass
								
		# Try to open the recieved data
		try:
			with open(stats_path + "tx_bytes") as file:
				# Try to read some data
				data = file.readline()
				
				# Check to see if data was read
				if not data == "":
					# Update the sent bytes
					self.current.up = int(data.strip())
		except IOError:
			pass
			
		# Update the total
		self.current.total = self.current.up + self.current.down
				
		return
		
	
	def connect(self, interface):
		"""Called upon connect, sets the interface and starts monitoring
		usage duration
		
		Parameters:
			interface, the interface that is being used for the connection
		"""
		
		# Remeber the interface
		self.interface = interface
		
		# Create a new record for the current connection
		self.current = JBMUsage.Record()
		
		# Get the last index
		if len(self.data) > 0:
			self.current.index = self.data[len(self.data) - 1].index + 1
		
		# Append the current record onto the data
		self.data.append(self.current)
		
	def disconnect(self):
		"""Saves and resets the current usage"""

		# Perform a final update
		self.__update_current()
		
		# Dereferance the current record
		self.current = None
		
		# Reset the interface
		self.interface = ""
		
		# Save the data
		self.save()
		
		# Reset the sub total
		self.sub_total = None

	def get_current(self, update=True):
		"""Returns the current usage updating it if required"""
		# Check to see if there is a current connection
		if self.current == None:
			return None
		
		# Update the usage if required
		if update:
			self.__update_current()
			
		# Format the current usage
		cur_form = self.format_record(self.current)
		
		# Change the date to be "Current"
		cur_form[0] = "Current"
		
		# Return the formated record
		return cur_form
	
	def get_total(self):
		"""Returns the total usage, calculating it if required"""
		
		# Check to see if the sub total needs to be recalculated
		if self.sub_total == None:
			# Create a new record to hold the subtotal
			self.sub_total = JBMUsage.Record()
			
			# Loop through the data
			for record in self.data:
				# Check to see if the record if the record is the
				#	current record
				if not record == self.current:
					# Add the record
					self.sub_total.add(record)
					
		
		# Create a temporary record to hold the total
		total = JBMUsage.Record()
		
		# Add the subtotal and the current to the total
		if not self.current == None:
			total.add(self.current)
		if not self.sub_total == None:
			total.add(self.sub_total)
			
		# Format the total
		total_form = self.format_record(total)
		
		# Change the date to be "Total"
		total_form[0] = "Total"
		
		# Return the total
		return total_form
		
