#!/usr/bin/python

#
# File: jbm_wvdial.py
#
# Description: Wrapper for wvdial for use with Jon's Broadband Manager
# 
# License:
#		jbm_wvdial.py
#		Copyright 2009 Jon Estey
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License version 2 as
#		published by the Free Software Foundation
#       
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#       
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.
#

# **************** Import ****************
import os
import threading
import Queue
from subprocess import *
from jbm_queue import *

# **************** Globals ****************

# **************** Classes ****************
class WVDial(threading.Thread):
	"""A class that runs wvdial in a seperate process while also monitering
	its output in a seperate thread"""
	
	class Thread(threading.Thread):
		def __init__(self, parent, output, wvdialrc):
			"""Initates the thread"""
			threading.Thread.__init__(self)
			
			self.wvdialrc = wvdialrc
			
			# Initiate varialbes
			self.parent = parent
			self.output = output
			
			self.interface = ""
			self.primary_dns = ""
			self.secondary_dns = ""
			self.local_ip = ""
			self.remote_ip = ""
		
			
		def run(self):
			"""Thread function.. dials the modem with wvdial and monitors
			output"""

			print "Dialer: Starting using {0}".format(self.wvdialrc)
			
			# Start wvdial
			wvdial = Popen("wvdial --config \"{0}\"".format(self.wvdialrc), shell=True, stdout=PIPE,
					stderr=STDOUT, bufsize=1)
			
			# Remember the subprocess (for termination)
			self.parent.wvdial = wvdial
			
			# Loop forever..
			while True:
				# Read data from wvdial
				data = wvdial.stdout.readline()
				
				# Ensure that data was read, an empty string means that
				#  the process has closed
				if data == "":
					break
					
				# Process the data that was read...

				# Remove white spaces
				data = data.strip()
				
				print "Dialer: '" + data + "'"
				
				# Try to match the output to known values
				if data.startswith("--> Using interface"):
					# Got the interface
					self.interface = data[20:].strip()
					
					# Connected, message the main thread
					self.output.put(JBMMessage(JBM_MSG_CONNECT, self.interface))
				if data.startswith("--> local  IP address"):
					# Got a local ip
					self.local_ip = data[22:].strip()
					
					# Message the main thread
					self.output.put(JBMMessage(JBM_MSG_IP, self.local_ip))
				elif data.startswith("--> remote IP address"):
					self.remote_ip = data[22:].strip()
					
					# Message the main thread
					self.output.put(JBMMessage(JBM_MSG_REM_IP, self.remote_ip))
				elif data.startswith("--> primary   DNS address"):
					self.primary_dns = data[26:].strip()
					
					# Message the main thread
					self.output.put(JBMMessage(JBM_MSG_DNS1, self.primary_dns))
				elif data.startswith("--> secondary DNS address"):
					self.secondary_dns = data[26:].strip()
					
					# Message the main thread
					self.output.put(JBMMessage(JBM_MSG_DNS2, self.secondary_dns))

			# Post the disconnected message
			self.output.put(JBMMessage(JBM_MSG_DISCONNECT))
			
			# Delete wvdial
			del wvdial
			self.parent.wvdial = None
			
			# Tell the parent the thread has died
			self.parent.thread_closed()
	
	def __init__(self, output_queue, wvdialrc, config):
		"""Initiates the class, message_queue provides a method of feedback
		for the thread"""
		
		# Remeber the messageing queue
		self.output = output_queue
		
		# Remember the config
		self.config = config
		self.wvdialrc = config.get("wvdialrc", "dialer", wvdialrc)
		
		# Initiate other properties
		self.thread = None
		self.wvdial = None
		
	def thread_closed(self):
		"""Called by the thread on exit, deletes the thread object"""
		
		del self.thread
		self.thread = None
	
	def make_rc(self):
		"""Builds a default wvdialrc"""
		
		# Try to open the file
		try:
			file = open(self.wvdialrc, "w")
		except:
			return
		
		# Load config settings
		port = self.config.get("port", "dialer", "/dev/ttyUSB2")
		apn = self.config.get("apn", "dialer", "Telstra.internet")
		baud = self.config.get("baud", "dialer", 7200000)
		phonenum = self.config.get("phone number","dialer", "*99#")
		
		# Write the default values
		file.write("""[Dialer Defaults]
baud = {baud}
modem = {port}
phone = {phone}

dial command = ATM1L3DT

init1 = ATZ
init2 = AT+CGDCONT=1,"IP","{apn}"
init3 = ATS30=0

stupid mode = on
auto dns = on
check dns = on
abort on no dialtone = off
isdn = off
ask password = off
idle seconds = 0
username = na
abort on busy = off
password = off
carrier check = off
auto reconnect = off
check def route = on
dial attempts = 1""".format(baud=baud, phone=phonenum, apn=apn, port=port))

		# Close the file
		file.close()
		
	def update_rc(self):
		"""Updates the wvdialrc file"""
		
		# Read in the wvdial rc
		try:
			# Open the file
			file = open(self.wvdialrc, "r")
			# Read all data
			data = file.read()
			# Close the file
			file.close()
		except:
			# Make default wvdialrc
			self.make_rc()
			
			return
			
		
		# Split the file into lines
		lines = data.split("\n")
		
		# Write the file
		try:
			# Open the file
			file = open(self.wvdialrc, "w")
			# Write all data
			for line in lines:
				if line.startswith("modem"):
					# Update modem line
					line = "modem = {0}".format(self.config.get("port","dialer","/dev/ttyUSB2"))
				elif line.startswith("init2"):
					# Update the apn
					line = "init2 = AT+CGDCONT=1,\"IP\",\"{0}\"".format(self.config.get("apn","dialer","Telstra.internet"))
				
				file.write(line + "\n")
				
			# Close the file
			file.close()
		except:
			return
		
		
	def dial(self):
		"""Dials the modem"""
		
		# Update the .wvdialrc if allowed
		if self.config.get("auto update rc","dialer",True):
			self.update_rc()
		
		# Start the thread
		if self.thread == None:
			self.thread = WVDial.Thread(self, self.output, self.wvdialrc)
			self.thread.start()
		
	def disconnect(self):
		"""Disconnect the modem"""
		
		# check for wvdial
		if self.wvdial == None:
			# Check to see if the thread is still present
			if self.thread == None:
				return
				
			# Delete the thread
			del self.thread
			self.thread = None
			
			return
		
		# Kill the subprocess
		self.wvdial.terminate()
		
		# Join the thread..
		self.thread.join(10.0)
		
		# Check to see if the thread is still alive
		if not self.thread == None and self.thread.isAlive():
			# Try to kill the process
			self.wvdial.kill()
			
			# Join the thread...
			self.thread.join(10.0)
			
			# Check to see if the thread is still alive...
			if self.thread.isAlive():
				# Raise an exception
				pass
				
		# Delete the thread object
		del self.thread
		self.thread = None

	
	
# **************** Main ****************

# Module testing code...
if __name__ == "__main__":
	from time import sleep
	
	print "Creating Output Queue"
	output = Queue.Queue()
	
	print "Creating wvdial object"
	wvdial = WVDial(output)
	
	print "Dialing..."
	wvdial.Dial()
	
	print "Processing Messages..."
	while True:
		try:
			msg = output.get(True, 20)
			print "Message ({0}): {1}".format(msg.type, msg.data)
		except Queue.Empty:
			break
		
	print "Disconnecting..."
	wvdial.Disconnect()
	
	print "Processing Messages..."
	while True:
		try:
			msg = output.get(True, 20)
			print "Message ({0}): {1}".format(msg.type, msg.data)
		except Queue.Empty:
			break
